<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Validator;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Customer\CustomerChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Dedi\SyliusCustomerPricingPlugin\Validator\Constraints\UniqueCustomerChannelPricing;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

class UniqueCustomerChannelPricingValidator extends ConstraintValidator
{
    public function __construct(
        private readonly ChannelPricingRepositoryInterface $channelPricingRepository
    )
    {

    }

    public function validate(mixed $value, Constraint $constraint)
    {
        if (!$value instanceof CustomerChannelPricingInterface) {
            throw new UnexpectedValueException($value, CustomerChannelPricingInterface::class);
        }

        if (!$constraint instanceof UniqueCustomerChannelPricing) {
            throw new UnexpectedValueException($constraint, UniqueCustomerChannelPricing::class);
        }

        if (null === $value->getProductVariant()
            || null === $value->getCustomer()
        ) {
            return;
        }

        $existing = $this->channelPricingRepository->findAllForCustomerForProductVariantAndChannel($value->getChannelCode(), $value->getProductVariant(), $value->getCustomer());

        $existing = array_filter($existing, static fn(CustomerChannelPricingInterface $item) => $item->getId() !== $value->getId());

        if (count($existing) > 0) {
            $this->context->buildViolation($constraint->customerChannelPricingAlreadyExistsMessage)
                ->addViolation();
        }
    }
}
